"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs-extra");
const path = require("path");
class Walker {
    constructor(modulePath) {
        this.prodPaths = new Set();
        this.rootModule = modulePath;
    }
    relativeModule(rootPath, moduleName) {
        return path.resolve(rootPath, 'node_modules', moduleName);
    }
    loadPackageJSON(modulePath) {
        return __awaiter(this, void 0, void 0, function* () {
            const pJPath = path.resolve(modulePath, 'package.json');
            if (yield fs.pathExists(pJPath)) {
                return yield fs.readJson(pJPath);
            }
            return null;
        });
    }
    loadProductionDependenciesForModuleInModule(moduleName, modulePath) {
        return __awaiter(this, void 0, void 0, function* () {
            let testPath = modulePath;
            let discoveredPath = null;
            let lastRelative = null;
            // Try find it while searching recursively up the tree
            while (!discoveredPath && this.relativeModule(testPath, moduleName) !== lastRelative) {
                lastRelative = this.relativeModule(testPath, moduleName);
                if (yield fs.pathExists(lastRelative)) {
                    discoveredPath = lastRelative;
                }
                else {
                    testPath = path.dirname(path.dirname(testPath));
                }
            }
            // If we can't find it the install is probably buggered
            if (!discoveredPath) {
                throw new Error(`Failed to locate module "${moduleName}" from "${modulePath}"`);
            }
            // If we can find it let's do the same thing for that module
            yield this.loadProductionDependenciesForModule(discoveredPath);
        });
    }
    loadProductionDependenciesForModule(modulePath) {
        return __awaiter(this, void 0, void 0, function* () {
            // We have already traversed this module
            if (this.prodPaths.has(modulePath))
                return;
            // Record this module as a production dependency
            this.prodPaths.add(modulePath);
            const pJ = yield this.loadPackageJSON(modulePath);
            // If the module doesn't have a package.json file it is probably a
            // dead install from yarn (they dont clean up for some reason)
            if (!pJ)
                return;
            // For every prod dep
            for (const moduleName in pJ.dependencies) {
                yield this.loadProductionDependenciesForModuleInModule(moduleName, modulePath);
            }
            // For every optional dep
            // (we do this to be safe but could be smarted about it later)
            for (const moduleName in pJ.optionalDependencies) {
                yield this.loadProductionDependenciesForModuleInModule(moduleName, modulePath);
            }
        });
    }
    loadProductionDependencies() {
        return __awaiter(this, void 0, void 0, function* () {
            this.prodPaths = new Set();
            yield this.loadProductionDependenciesForModule(this.rootModule);
            return this.prodPaths;
        });
    }
    pruneModule(modulePath) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.prodPaths.has(modulePath)) {
                const nodeModulesPath = path.resolve(modulePath, 'node_modules');
                if (!(yield fs.pathExists(nodeModulesPath)))
                    return;
                for (const subModuleName of yield fs.readdir(nodeModulesPath)) {
                    yield this.pruneModule(path.resolve(nodeModulesPath, subModuleName));
                }
            }
            else {
                yield fs.remove(modulePath);
            }
        });
    }
    prune() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.loadProductionDependencies();
            yield this.pruneModule(this.rootModule);
        });
    }
}
exports.Walker = Walker;
//# sourceMappingURL=Walker.js.map